@@ -24,14 +24,13 @@ import functools
 import time
 import calendar
 
-import six
-
 from botocore.exceptions import NoCredentialsError
 from botocore.utils import normalize_url_path, percent_encode_sequence
 from botocore.compat import HTTPHeaders
 from botocore.compat import quote, unquote, urlsplit, parse_qs
 from botocore.compat import urlunsplit
 from botocore.compat import encodebytes
+from botocore.compat import six
 
 logger = logging.getLogger(__name__)
 
