@@ -13,10 +13,11 @@
 
 import copy
 import datetime
-import six
 import sys
 import inspect
 
+from botocore.vendored import six
+
 
 if six.PY3:
     from six.moves import http_client
